import os

# import json
# import time
# import sys

from binarycpython.utils.grid import Population
from binarycpython.utils.functions import get_help_all, get_help, create_hdf5



#########################################################
# This file serves as an example for running a population.
# The use of help(<function>) is a good way to inspect what parameters are there to use
#########################################################

# Create population object
example_pop = Population()

# If you want verbosity, set this before other things
example_pop.set(verbose=1)

# Setting values can be done via .set(<parameter_name>=<value>)
# Values that are known to be binary_c_parameters are loaded into bse_options.
# Those that are present in the default grid_options are set in grid_options
# All other values that you set are put in a custom_options dict
example_pop.set(
    #
    M_1=10,  # bse_options
    separation=0,  # bse_options
    orbital_period=4580,  # bse_options
    max_evolution_time=15000,  # bse_options
    eccentricity=0.02,  # bse_options
    #
    amt_cores=1,  # grid_options
    #
    data_dir=os.path.join(
        os.environ["BINARYC_DATA_ROOT"], "example_python"
    ),  # custom_options
    base_filename="example_pop.dat",  # custom_options
)

# Custom logging
# TODO: show different ways
example_pop.set(
    C_auto_logging={
        "MY_HEADER_LINE": ["star[0].mass", "star[1].mass", "model.probability"]
    }
)
example_pop.set(
    C_logging_code='Printf("MY_STELLAR_DATA time=%g mass=%g radius=%g\\n", stardata->model.time, stardata->star[0].mass, stardata->star[0].radius);'
)

# TODO: show when reading from file
example_pop.set_custom_logging() # TODO: remove this one

# Adding grid variables:
example_pop.add_grid_variable(
    name="lnm1",
    longname="log primary mass",
    valuerange=[10, 20],
    resolution="10",
    spacingfunc="np.linspace(0.213, 10.2, 10)",
    precode="M_1=math.exp(lnm1)",
    probdist="flat(M_1)",
    # probdist='self.custom_options["extra_prob_function"](M_1)',
    dphasevol="",
    parameter_name="M_1",
    condition="",
)

# Exporting of all the settings can be done with .export_all_info()
# on default it exports everything, but can be supressed by turning it off:
#   population settings (bse_options, grid_options, custom_options), turn off with include_population_settings=False
#   binary_c_defaults (all the commandline arguments that binary c accepts, and their defaults). turn off with include_binary_c_defaults=False
#   include_binary_c_version_info (all the compilation info, and information about the compiled parameters), turn off with include_binary_c_version_info=False
#   include_binary_c_help_all (all the help information for all the binary_c parameters), turn off with include_binary_c_help_all=Fase
# On default it will write this to the custom_options['data_dir'], but that can be overriden by setting use_datadir=False and providing an outfile=<>
example_pop.export_all_info()

# Creating a parsing function
# TODO: add example of setting up a parsing function

# Executing a single system
# TODO: add example of running a single system

# Executing a population
# TODO: add example of running a population

# Wrapping up the results to an hdf5 file can be done by using the create_hdf5(<directory containing data and settings>)
# This function takes the settings file (ending in _settings.json) and the data files (ending in .dat) from the data_dir
# and packing them into an hdf5 file, which is then written into the same data_dir directory
create_hdf5(example_pop.custom_options["data_dir"])